#include "datebang.h"

extern "C" {
__declspec( dllexport ) int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath);
__declspec( dllexport ) int quitModule(HINSTANCE dll);
__declspec( dllexport ) BOOL WINAPI DllMain(HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpReserved );
}

int AtVarProc(char *dest, int maxchars, LPCSTR src, LPARAM lParam);

void BangExec(HWND caller, LPCSTR args)
{
  char buffer[MAX_LINE_LENGTH];
  LSReplaceAtVars(buffer, MAX_LINE_LENGTH, args, AtVarProc, 0);
  LSExecute(caller,buffer,SW_SHOWNA);
}

int initModuleEx(HWND ParentWnd, HINSTANCE dll, LPCSTR szPath)
{
  AddBangCommand("!DATEEXEC",BangExec);
  return 0;
}

int quitModule(HINSTANCE dll)
{
  RemoveBangCommand("!DATEEXEC");
  return 0;
}




int IntVar(char *dest, int maxchars, const char *format, int width, unsigned value)
{
  if (maxchars < width) return 0;
  return sprintf(dest,format,value);
}

int TimeVar(char *dest, int maxchars, LPCSTR src, LPARAM lParam, bool bUTC)
{
  SYSTEMTIME st;  

  if (bUTC) GetSystemTime(&st);
  else GetLocalTime(&st);

  switch (*src++)
  {
  case 'a': /* 0/1 for am/pm */ 
    if (maxchars<2) return 0;
    *dest = st.wHour > 11 ? '1' : '0'; return 1;
  case 'h': /* 0-23 */ 
    return IntVar(dest,maxchars,"%d",3,st.wHour);
  case 'H': /* 00-23 */ 
    return IntVar(dest,maxchars,"%02d",3,st.wHour);
  case 'i': /* 1-12 */ 
    return IntVar(dest,maxchars,"%d",3,(st.wHour%12)+1);
  case 'I': /* 01-12 */ 
    return IntVar(dest,maxchars,"%02d",3,(st.wHour%12)+1);
  case 'l': /* milliseconds */ 
    return IntVar(dest,maxchars,"%d",4,st.wMilliseconds);
  case 'L': /* padded to ? */ 
    return IntVar(dest,maxchars,"%03d",4,st.wMilliseconds);
  case 'm': /* 0-59 */ 
    return IntVar(dest,maxchars,"%d",3,st.wMinute);
  case 'M': /* 00-59 */ 
    return IntVar(dest,maxchars,"%02d",3,st.wMinute);
  case 's': /* 0-59 */ 
    return IntVar(dest,maxchars,"%d",3,st.wSecond);
  case 'S': /* 00-59 */ 
    return IntVar(dest,maxchars,"%02d",3,st.wSecond);
  case 't': /* GetTickCount->%d */ 
    return IntVar(dest,maxchars,"%d",8,GetTickCount());
  }
  return 0;
}

int DateVar(char *dest, int maxchars, LPCSTR src, LPARAM lParam, bool bUTC)
{
  SYSTEMTIME st;
  int dayofyear,weekofyear;
  bool isleapyear;
  const static int dayspermonth[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

  if (bUTC) GetSystemTime(&st);
  else GetLocalTime(&st);

  isleapyear =
    (st.wYear%4 == 0) /* year divisible by 4 */ 
    && ((st.wYear%100 != 0) /* and not divisible by 100 */ 
      || (st.wYear%400 == 0)); /* unless divisible by 400 */ 

  dayofyear = st.wDay; /* day of month */ 
  for (weekofyear=0; weekofyear<st.wMonth; weekofyear++)
    dayofyear += dayspermonth[weekofyear]; /* + days in preceding months */ 
  if (isleapyear && st.wMonth>2) ++dayofyear; /* adjust for feb 29 */ 

  weekofyear = 1+(dayofyear-1)/7;
  /* weeks start on whatever day of the week jan 1 is */ 
  /* maybe that's wrong, but too bad */ 

  switch (*src++)
  {
  case 'a': /* day of week 0-7 */ 
    return IntVar(dest,maxchars,"%d",2,st.wDayOfWeek);
  case 'd': /* day of month 1-31 */ 
    return IntVar(dest,maxchars,"%d",3,st.wDay);
  case 'D': /* day of month 01-31 */ 
    return IntVar(dest,maxchars,"%02d",3,st.wDay);
  case 'e': /* day of year 1-365 */ 
    return IntVar(dest,maxchars,"%d",4,dayofyear);
  case 'E': /* day of year 001-365 */ 
    return IntVar(dest,maxchars,"%03d",4,dayofyear);
  case 'l': /* 1 if leap year, 0 if not */ 
    if (maxchars<2) return 0;
    *dest = isleapyear ? '1' : '0'; return 1;
  case 'm': /* month 1-12 */ 
    return IntVar(dest,maxchars,"%d",3,st.wMonth);
  case 'M': /* month 01-12 */ 
    return IntVar(dest,maxchars,"%02d",3,st.wMonth);
  case 'w': /* week of year 1-52 */ 
    return IntVar(dest,maxchars,"%d",3,weekofyear);
  case 'W': /* week of year 01-52 */ 
    return IntVar(dest,maxchars,"%02d",3,weekofyear);
  case 'y': /* year 03 */ 
    return IntVar(dest,maxchars,"%02d",3,st.wYear%100);
  case 'Y': /* year 2003 */ 
    return IntVar(dest,maxchars,"%d",3,st.wYear);
  }

  return 0;
}

int AtVarProc(char *dest, int maxchars, LPCSTR src, LPARAM lParam)
{
  switch (*src++)
  {
    /* #, k, K are reserved */ 
  case '_':
    switch (*src++)
    {
    case 'n': /* name */ 
    case 'N':
      if (maxchars<9) return 0;
      strcpy(dest,"DateBang");
      return 8;
    case 'v': /* version */ 
    case 'V':
      if (maxchars<4) return 0;
      strcpy(dest,"1.0");
      return 3;
    case 'a': /* author */ 
    case 'A':
      if (maxchars<9) return 0;
      strcpy(dest,"RabidCow");
      return 8;
    }
    break;

  case 't': return TimeVar(dest,maxchars,src,lParam,false); /* local */ 
  case 'T': return TimeVar(dest,maxchars,src,lParam,true); /* UTC */ 
  case 'd': return DateVar(dest,maxchars,src,lParam,false); /* local */ 
  case 'D': return DateVar(dest,maxchars,src,lParam,true); /* UTC */ 
  case 'f': /* GetTimeFormat */ 
  case 'F': /* needs to handle all sorts of crud */ 
    return 0;
  }
  return 0;
}
